@@ -1,5 +1,6 @@
 import datetime
 import io
+import logging
 import os
 import tempfile
 import threading
@@ -15,29 +16,33 @@ from satella.coding.concurrent import PeekableQueue
 from satella.coding.structures import DirtyDict
 
 from .api import RequestsAPI
-from .certificate import get_device_info
+from .certificate import get_device_info, get_root_cert, get_dev_ca_cert
 from .slave import SlaveDevice
 from ..baob import BAOB
 from ..basics import DeviceInfo, Environment, StorageLevel
 from ..exceptions import ResponseError, UnavailableError
 from ..extras import BaseSensorDatabase, BaseEventDatabase, BaseMacroDatabase, \
-    BasePathpointDatabase, BaseMetadataDatabase, BaseBAOBDatabase, BaseArchivesDatabase
+    BasePathpointDatabase, BaseMetadataDatabase, BaseBAOBDatabase, BaseArchivesDatabase, \
+    BaseSensorWriteDatabase, BasePredicateDatabase
 from ..extras.arch_database.in_memory import InMemoryArchivesDatabase
 from ..extras.baob_database.memory import InMemoryBAOBDatabase
 from ..extras.event_database import InMemoryEventDatabase
 from ..extras.macros_database.in_memory import InMemoryMacroDatabase
 from ..extras.metadata_database import InMemoryMetadataDatabase
 from ..extras.pp_database.in_memory import InMemoryPathpointDatabase
+from ..extras.pred_database.in_memory import InMemoryPredicateDatabase
+from ..extras.sensor_write_database.in_memory import InMemorySensorWriteDatabase
 from ..extras.sensors_database.in_memory import InMemorySensorDatabase
 from ..metadata import PlainMetadata
 from ..pathpoint import Pathpoint, ReparsePathpoint
 from ..pathpoint.orders import Section, MessageOrder
 from ..predicate import BaseStatistic, Event, Color
-from ..sensor import Sensor, fqtsify
+from ..predicate.registration import CollectionOfStatistics, StatisticRegistration
+from ..predicate.undefined import UndefinedStatistic
+from ..sensor import Sensor, fqtsify, SensorWriteEvent
 from ..threads import OrderExecutorThread, CommunicatorThread, ArchivingAndMacroThread, \
     LogPublisherThread
-import logging
-
+from ..threads.communicator import PREDICATE_SYNC_INTERVAL
 
 logger = logging.getLogger(__name__)
 
@@ -53,6 +58,9 @@ class SMOKDevice(Closeable, metaclass=ABCMeta):
 
     :param cert: either a path to or a file-like object containing the device certificate
     :param priv_key: either a path to or a file-like object containing the device private key
+    :param allow_sync: whether to allow outbound TCP communication to automatically synchronize
+        background. Note that if you explicitly call a method that calls TCP this does not apply.
+        Can be lated changed.
     :param evt_database: custom event database. Providing a string defaults to path where predicate
         data will be persisted. Since events are persistent, a store needs to be given.
     :param pp_database: custom pathpoint value database. Default value of None defaults to an
@@ -65,19 +73,24 @@ class SMOKDevice(Closeable, metaclass=ABCMeta):
         in-memory implementation
     :param baob_database: custom BAOB database. Default value of None will result in an
         in-memory implementation
+    :param pred_database: custom Predicate database. Default value of None will result in an
+        in-memory implementation
     :param arch_database: custom archives database. Default value of None will result in an
         in-memory implementation
+    :param sensor_write_database: custom sensor write database. Default value of None will result in
+        an in-memory implementation
     :param dont_obtain_orders: if set to True, this SMOKDevice won't poll for orders.
         This also implies dont_do_baobs. It is a ValueError to set this while setting
         dont_bo_baobs to False.
     :param dont_do_pathpoints: if set to True, this SMOKDevice won't support pathpoints
         or sensors.
+    :param dont_sync_sensor_writes: if set to True, sensor writes won't be synced
     :param dont_do_baobs: if set to True, this SMOKDevice won't care about BAOBs.
     :param dont_do_macros: if set to True, this SMOKDevice won't take care of the macros
     :param dont_do_predicates: if set to True, this SMOKDevice won't do predicates
     :param dont_do_archives: if set to True, this SMOKDevice won't do archiving
     :param startup_delay: amount of seconds to wait after creation for CommunicatorThread to
-        start talking
+        start talking and OrderExecutorThread to start grabbing orders
     :param cache_metadata_for: amount of seconds to cache downloaded metadata entry.
         Ie no attempt to download them from the server again will be made in that many
         seconds since the download.
@@ -89,8 +102,15 @@ class SMOKDevice(Closeable, metaclass=ABCMeta):
     won't be started.
     If dont_obtain_orders is True, then order executor and order getter threads won't be started.
 
-    :ivar device_id: device ID of this device
-    :ivar environment: environment of this device
+    Every method is supposed to be called from user threads (SMOKDevice's service threads included),
+    unless noted otherwise.
+
+    :ivar device_id: device ID of this device (str)
+    :ivar allow_sync: whether to allow background synchronization (bool).
+        This can be changed by setting it directly to correct value. Setting this to False
+        will help you save some bandwidth. Note that log upload will be put on hold, until it
+        overflows the buffer. In that case, earlier submitted logs will be thrown on the floor
+        to make room for new entries.
     :ivar pathpoints: a dictionary, keying pathpoint names to their instances
     :ivar url: base URL for the API calls, without the trailing slash
     :ivar metadata: plain metadata for this device
@@ -154,8 +174,12 @@ class SMOKDevice(Closeable, metaclass=ABCMeta):
                  meta_database: tp.Optional[BaseMetadataDatabase] = None,
                  sensor_database: tp.Optional[BaseSensorDatabase] = None,
                  baob_database: tp.Optional[BaseBAOBDatabase] = None,
+                 pred_database: tp.Optional[BasePredicateDatabase] = None,
                  arch_database: tp.Optional[BaseArchivesDatabase] = None,
+                 sensor_write_database: tp.Optional[BaseSensorWriteDatabase] = None,
+                 allow_sync: bool = True,
                  dont_obtain_orders: bool = False,
+                 dont_sync_sensor_writes: bool = False,
                  dont_do_macros: bool = False,
                  dont_do_predicates: bool = False,
                  dont_do_pathpoints: bool = False,
@@ -167,24 +191,35 @@ class SMOKDevice(Closeable, metaclass=ABCMeta):
         self.cache_metadata_for = cache_metadata_for
         self.dont_do_predicates = dont_do_predicates
         self.dont_do_pathpoints = dont_do_pathpoints
+        self.dont_sync_sensor_writes = dont_sync_sensor_writes
         self.pp_database = pp_database or InMemoryPathpointDatabase()
         self.baobs_loaded = False
+        self.allow_sync = allow_sync
         if isinstance(evt_database, str):
             self.evt_database = InMemoryEventDatabase(evt_database)
         else:
             self.evt_database = evt_database
         self.macros_database = macro_database or InMemoryMacroDatabase()
         self.meta_database = meta_database or InMemoryMetadataDatabase()
+        self.pred_database = pred_database or InMemoryPredicateDatabase()
         self.sensor_database = sensor_database or InMemorySensorDatabase()
         self.sensor_database.on_register(self)
         self.arch_database = arch_database or InMemoryArchivesDatabase()
         self.baob_database = baob_database or InMemoryBAOBDatabase()
+        self.sensor_write_database = sensor_write_database or InMemorySensorWriteDatabase()
         self.metadata = PlainMetadata(self)
         self.ready_lock = threading.Lock()
         self.ready_lock.acquire()
+
         self.predicates = {}  # type: tp.Dict[str, BaseStatistic]
+
+        # Load cached predicates
+        for predicate in self.pred_database.get_all_predicates():
+            udf = UndefinedStatistic(device=self, **predicate)
+            self.predicates[udf.predicate_id] = udf
+
         self._timezone = None
-        self.predicate_classes = {}  # type: tp.Dict[str, tp.Type[BaseStatistic]]
+        self.statistic_registration = CollectionOfStatistics()
         self._statistics_updated = False
         self.pathpoints = DirtyDict()  # type: tp.Dict[str, Pathpoint]
         self.temp_file_for_cert = None
@@ -204,6 +239,7 @@ class SMOKDevice(Closeable, metaclass=ABCMeta):
         with open(cert, 'rb') as fin:
             cert_data = fin.read()
 
+        self.cert_data = cert_data
         dev_id, env = get_device_info(cert_data)
         self.device_id = dev_id  # type: str
         self.environment = env  # type: Environment
@@ -225,17 +261,39 @@ class SMOKDevice(Closeable, metaclass=ABCMeta):
         self.dont_do_baobs = dont_do_baobs
         if not dont_obtain_orders or not dont_do_predicates or not dont_do_pathpoints or \
                 not dont_do_baobs:
-            self.executor = OrderExecutorThread(self, self._order_queue, self.pp_database).start()
+            self.executor = OrderExecutorThread(self, self._order_queue, self.pp_database,
+                                                startup_delay).start()
             self.getter = CommunicatorThread(self, self._order_queue, self.pp_database,
                                              dont_obtain_orders,
                                              dont_do_baobs,
                                              dont_do_pathpoints,
-                                             dont_do_predicates, startup_delay).start()
+                                             dont_do_predicates,
+                                             dont_sync_sensor_writes, startup_delay).start()
         else:
             self.executor = None
             self.getter = None
         self.log_publisher = LogPublisherThread(self).start()
 
+    def log_sensor_write(self, sw: SensorWriteEvent):
+        """
+        Log that a sensor has been written and enqueue it for cloud upload
+
+        :param sw: sensor write event to upload
+        """
+        self.sensor_write_database.add_sw(sw)
+
+    def reset_predicates(self):
+        """
+        Clear all loaded predicates and force a renew of loading.
+
+        Discards all currently loaded Predicate instances. Currently registered statistics will
+        remain registered.
+
+        .. warning:: Currently requires Internet access to restore predicates
+        """
+        self.predicates = {}
+        self.getter.last_predicates_synced = time.monotonic() - PREDICATE_SYNC_INTERVAL
+
     def get_baob(self, key: str) -> BAOB:
         """
         Retrieve given BAOB
@@ -273,8 +331,10 @@ class SMOKDevice(Closeable, metaclass=ABCMeta):
         for i in range(3):
             try:
                 self.api.post('/v1/device/orders/message/' + order.uuid)
-            except ResponseError:
-                pass
+                self.on_successful_sync()
+            except ResponseError as e:
+                if e.is_no_link():
+                    self.on_failed_sync()
 
     @property
     def timezone(self) -> pytz.timezone:
@@ -326,6 +386,18 @@ class SMOKDevice(Closeable, metaclass=ABCMeta):
         with self.ready_lock:
             yield from self.sensor_database.get_all_sensors()
 
+    def on_baob_updated(self, baob_name: str) -> None:
+        """
+        Called by CommunicatorThread after given BAOB was updated.
+
+        After this is called, the new BAOB can be successfully loaded and it's new
+        contents will be retrieved.
+
+        This is not invoked during the first synchronization.
+
+        :param baob_name: name of the BAOB that was just downloaded from the server
+        """
+
     def get_sensor(self, tag_set: tp.Union[tp.Set[str], str]) -> Sensor:
         """
         Return a sensor
@@ -387,7 +459,9 @@ class SMOKDevice(Closeable, metaclass=ABCMeta):
 
     def execute(self, *secs: Section) -> None:
         """
-        Schedule sections to be executed
+        Schedule sections to be executed.
+
+        To be invoked by any thread. Use this to inject sections into device's execution loop.
 
         :param secs: sections to be executed, in that order, unless they're joinable
         """
@@ -415,7 +489,8 @@ class SMOKDevice(Closeable, metaclass=ABCMeta):
         self.register_pathpoint(pp)
         return pp
 
-    def register_statistic(self, stat: tp.Type[BaseStatistic]) -> None:
+    def register_statistic(self, stat: tp.Type[BaseStatistic],
+                           predicate: tp.Callable[[str, dict], bool]) -> StatisticRegistration:
         """
         Register a new statistic.
 
@@ -423,13 +498,17 @@ class SMOKDevice(Closeable, metaclass=ABCMeta):
         instances will be created for them shortly by the communicator thread.
 
         :param stat: a class (not an instance) to register
-
+        :param predicate: a callable taking two arguments: statistic name and it's configuration.
+            The callable should return whether to apply stat to this predicate
+        :return: a Registration object. Can be later cancelled.
         :raise UnavailableError: SMOKDevice was launched in a no-predicate mode
         """
         if self.dont_do_predicates:
             raise UnavailableError('SMOKDevice was launched without predicates')
         assert issubclass(stat, BaseStatistic), 'Not a subclass of BaseStatistic!'
-        self.predicate_classes[stat.statistic_name] = stat
+        reg = StatisticRegistration(predicate, stat)
+        self.statistic_registration.add(reg)
+        return reg
 
     def register_pathpoint(self, pp: Pathpoint) -> None:
         """
@@ -450,23 +529,25 @@ class SMOKDevice(Closeable, metaclass=ABCMeta):
             pp.device = weakref.proxy(self)
             self.pathpoints[pp.name] = pp
 
-    def execute_sysctl(self, op_name: str, op_args: str) -> bool:
+    def execute_sysctl(self, op_type: str, op_args: str) -> bool:
         """
         Called by executor thread upon receiving a request to execute a particular SysctlOrder
 
         Handles commonly defined BAOBs. Extend to implement custom sysctls.
         Sysctl orders are user-defined.
 
-        :param op_name: name of operation to execute
+        :param op_type: type of operation to execute
         :param op_args: argument of the operation to execute.
         :return: whether this command was recognized and acted upon
         """
-        if op_name in ('baob-updated', 'baob-created'):
+        if self.dont_do_baobs:
+            return False
+        if op_type in ('baob-updated', 'baob-created'):
             if self.getter is not None:
                 self.getter.last_baob_synced = 0
                 self.getter.data_to_update.notify()
             return True
-        elif op_name == 'baob-deleted':
+        elif op_type == 'baob-deleted':
             self.baob_database.delete_baob(op_args)
             return True
         return False
@@ -511,13 +592,47 @@ class SMOKDevice(Closeable, metaclass=ABCMeta):
         """
         Obtain information about the device.
 
+        Note that this will result in :meth:`~smok.client.SMOKDevice.on_failed_sync`
+        or :meth:`~smok.client.SMOKDevice.on_successful_sync` being called.
+
         :return: current device information
         :raises ResponseError: server responded (or not) with an invalid message
         """
-        resp = DeviceInfo.from_json(self.api.get('/v1/device'))
+        try:
+            resp = DeviceInfo.from_json(self.api.get('/v1/device'))
+        except ResponseError as e:
+            if e.is_no_link():
+                self.on_failed_sync()
+            raise
+        self.on_successful_sync()
         self._timezone = resp.timezone
         return resp
 
+    def on_successful_sync(self, *args, **kwargs) -> None:
+        """
+        Called by CommunicatorThread each time a part of the system synchronizes correctly with the
+        server. Can be used to implement link liveness detector.
+
+        args and kwargs left for future extendability.
+
+        Please note that if you set allow_sync to False this won't be called, as the client won't
+        try to talk to the server. The only chance is getting synces manually, as via
+        :meth:`~smok.client.SMOKDevice.get_device_info`
+        """
+
+    def on_failed_sync(self, *args, **kwargs) -> None:
+        """
+        Called by CommunicatorThread each time a part of the system fails to synchronize by
+        receiving no response at all from the server. Can be used to implement link liveness
+        detector.
+
+        args and kwargs left for future extendability
+
+        Please note that if you set allow_sync to False this won't be called, as the client won't
+        try to talk to the server. The only chance is getting synces manually, as via
+        :meth:`~smok.client.SMOKDevice.get_device_info`
+        """
+
     def get_local_time(self) -> datetime.datetime:
         """
         Return current local time on target culture context
@@ -531,3 +646,15 @@ class SMOKDevice(Closeable, metaclass=ABCMeta):
         local_time = utc_time.astimezone(tz)
 
         return local_time
+
+    @property
+    def cert_chain(self) -> bytes:
+        """
+        Return your own certificate chain, finishing at SMOK CA certificate
+
+        :return: certificate chain in PEM format
+        """
+        my_dat = self.cert_data
+        dev_ca_dat = get_dev_ca_cert()
+        root_ca_dat = get_root_cert()
+        return b''.join((my_dat, dev_ca_dat, root_ca_dat))
