@@ -1,5 +1,5 @@
-import typing as tp
 import queue
+import typing as tp
 
 from satella.coding import queue_get
 from satella.coding.concurrent import TerminableThread
@@ -7,8 +7,8 @@ from satella.coding.decorators import retry
 
 from ..exceptions import ResponseError
 
-
 MAX_SYNC_AT_ONCE = 50
+MAX_LOG_BUFFER_SIZE = 20000
 
 
 class LogPublisherThread(TerminableThread):
@@ -25,14 +25,27 @@ class LogPublisherThread(TerminableThread):
 
     @queue_get('queue', timeout=5)
     def loop(self, msg):
+        while not self.device.allow_sync:
+            while self.queue.qsize() > MAX_LOG_BUFFER_SIZE:
+                self.queue.get()
+            self.safe_sleep(10)
+        if self._terminating:
+            return
         msgs = self.get_all_messages(msg)
         self.sync(msgs)
 
     @retry(3, exc_classes=ResponseError)
     def sync(self, lst: tp.List[dict]):
-        self.device.api.put('/v1/device/device_logs', json=lst, timeout=20)
+        try:
+            self.device.api.put('/v1/device/device_logs', json=lst, timeout=20)
+            self.device.on_successful_sync()
+        except ResponseError as e:
+            if e.is_no_link():
+                self.device.on_failed_sync()
+            raise
 
     def cleanup(self):
-        while self.queue.qsize() > 0:
-            msgs = self.get_all_messages(self.queue.get())
-            self.sync(msgs)
+        if self.device.allow_sync:
+            while self.queue.qsize() > 0:
+                msgs = self.get_all_messages(self.queue.get())
+                self.sync(msgs)
